@@ -1,20 +1,18 @@
 mod tag;
 
 use self::tag::Tag;
-use crate::{SyntaxKind, SyntaxNode, SyntaxToken};
+use crate::{SyntaxKind, SyntaxNode, SyntaxToken, TextRange, TreeConfig};
 use std::marker::PhantomData;
 use std::sync::atomic::{AtomicU32, Ordering};
 use std::{fmt, slice};
-use text_size::TextRange;
 
 /// `SyntaxTree` owns the syntax tree allocation.
 /// To construct a tree, see [`SyntaxBuilder`].
 /// To access its contents, see [`SyntaxTree::root`].
 ///
-/// `SyntaxTree`, like all other `Syntax*` types, is generic over `K`
-/// which must implement [`SyntaxKind`].
-/// This type parameter allows the kinds of nodes and tokens
-/// to be converted between a raw concrete type and a custom enum.
+/// `SyntaxTree`, like all other `Syntax*` types, is generic over a [`TreeConfig`],
+/// which specifies how the kinds of nodes and tokens
+/// can be converted between the library consumer’s custom enum and a raw concrete type.
 ///
 /// # Format
 ///
@@ -85,22 +83,22 @@ use text_size::TextRange;
 /// Due to `u16::MAX` being dedicated to *finish node*,
 /// we must prohibit a kind of fifteen 1s to avoid ambiguity.
 /// Thus, the highest allowed kind is `0b0111_1111_1111_1110`.
-pub struct SyntaxTree<K> {
+pub struct SyntaxTree<C> {
     data: Box<[u8]>,
-    phantom: PhantomData<K>,
+    phantom: PhantomData<C>,
 }
 
 /// This type is used to construct a [`SyntaxTree`].
 ///
 /// Due to the custom in-memory format used for [`SyntaxTree`],
 /// the text of your entire input must be provided up-front in [`SyntaxBuilder::new`].
-pub struct SyntaxBuilder<K> {
+pub struct SyntaxBuilder<C> {
     data: Vec<u8>,
     is_root_set: bool,
     current_len: u32,
     start_node_idxs: Vec<usize>,
     nesting: u32,
-    phantom: PhantomData<K>,
+    phantom: PhantomData<C>,
 }
 
 pub(crate) const START_NODE_SIZE: u32 = 2 + 4 + 4 + 4;
@@ -111,7 +109,7 @@ const FINISH_NODE_IDX_PLACEHOLDER: u32 = 0;
 
 static CURRENT_TREE_ID: AtomicU32 = AtomicU32::new(0);
 
-impl<K: SyntaxKind> SyntaxBuilder<K> {
+impl<C: TreeConfig> SyntaxBuilder<C> {
     /// Constructs a new empty `SyntaxBuilder` with the provided source text.
     pub fn new(text: &str) -> Self {
         Self::with_capacity(text, 0, 0, 0)
@@ -129,7 +127,8 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
         add_tokens: usize,
         finish_nodes: usize,
     ) -> Self {
-        debug_assert!(K::LAST <= Tag::MAX_KIND);
+        debug_assert!(C::NodeKind::LAST <= Tag::MAX_KIND);
+        debug_assert!(C::TokenKind::LAST <= Tag::MAX_KIND);
         assert!(text.len() < u32::MAX as usize);
 
         let id = CURRENT_TREE_ID.fetch_add(1, Ordering::SeqCst);
@@ -160,7 +159,7 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
     ///
     /// - if you have finished creating a root node and try to create another
     #[inline(always)]
-    pub fn start_node(&mut self, kind: K) {
+    pub fn start_node(&mut self, kind: C::NodeKind) {
         if self.is_root_set {
             assert_ne!(self.nesting, 0, "root node already created");
         } else {
@@ -174,7 +173,7 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
         self.data.reserve(START_NODE_SIZE as usize);
         unsafe {
             let ptr = self.data_end_ptr();
-            (ptr as *mut Tag).write_unaligned(Tag::start_node(kind));
+            (ptr as *mut Tag).write_unaligned(Tag::start_node::<C>(kind));
             (ptr.add(2) as *mut u32).write_unaligned(FINISH_NODE_IDX_PLACEHOLDER);
             (ptr.add(6) as *mut u32).write_unaligned(self.current_len);
             (ptr.add(10) as *mut u32).write_unaligned(self.current_len);
@@ -189,7 +188,7 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
     /// - if you try to add a token before starting a node
     /// - if the provided range is out of bounds of the original input text
     #[inline(always)]
-    pub fn add_token(&mut self, kind: K, range: TextRange) {
+    pub fn add_token(&mut self, kind: C::TokenKind, range: TextRange) {
         assert!(self.nesting > 0, "cannot add token before starting node");
         assert!(
             u32::from(range.end()) <= self.text_len(),
@@ -204,7 +203,7 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
         self.data.reserve(ADD_TOKEN_SIZE as usize);
         unsafe {
             let ptr = self.data_end_ptr();
-            (ptr as *mut Tag).write_unaligned(Tag::add_token(kind));
+            (ptr as *mut Tag).write_unaligned(Tag::add_token::<C>(kind));
             (ptr.add(2) as *mut u32).write_unaligned(start);
             (ptr.add(6) as *mut u32).write_unaligned(end);
             self.data.set_len(self.data.len() + ADD_TOKEN_SIZE as usize);
@@ -251,7 +250,7 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
     ///
     /// - if no nodes have been created
     /// - if there are nodes which have not been finished
-    pub fn finish(self) -> SyntaxTree<K> {
+    pub fn finish(self) -> SyntaxTree<C> {
         let Self { data, is_root_set, current_len: _, start_node_idxs: _, nesting, phantom: _ } =
             self;
 
@@ -272,22 +271,35 @@ impl<K: SyntaxKind> SyntaxBuilder<K> {
     }
 }
 
-impl<K: SyntaxKind> SyntaxTree<K> {
+impl<C: TreeConfig> SyntaxTree<C> {
     /// Returns the root node of this tree.
-    pub fn root(&self) -> SyntaxNode<K> {
-        SyntaxNode::new(self.root_idx(), self.id())
+    pub fn root(&self) -> SyntaxNode<C> {
+        unsafe { SyntaxNode::new(self.root_idx(), self.id()) }
     }
 
     /// Returns an iterator over the events stored in this tree.
     ///
+    /// The difference between this method and [`SyntaxTree::raw_events`] is that
+    /// this method returns [`SyntaxNode`]s and [`SyntaxToken`]s,
+    /// while [`SyntaxTree::raw_events`] returns the data actually stored in the tree.
+    pub fn events(&self) -> impl Iterator<Item = Event<C>> + '_ {
+        Events { idx: self.root_idx(), tree: self }
+    }
+
+    /// Returns an iterator over the raw events stored in this tree.
+    ///
+    /// As compared to [`SyntaxTree::events`],
+    /// this method emits the data actually stored in the tree,
+    /// as opposed to handles to that data ([`SyntaxNode`]s and [`SyntaxToken`]s).
+    ///
     /// This method does not compute any more information
     /// than what is stored in the tree.
-    /// The only difference between the [`Event`]s returned by this method
+    /// The only difference between the [`RawEvent`]s returned by this method
     /// and what is stored inside the tree
     /// is that the events returned by this method are fixed-length and typed,
     /// while the tree’s internal storage is variable-length and untyped.
-    pub fn events(&self) -> impl Iterator<Item = Event<K>> + '_ {
-        Events { idx: self.root_idx(), tree: self }
+    pub fn raw_events(&self) -> impl Iterator<Item = RawEvent<C>> + '_ {
+        RawEvents { idx: self.root_idx(), tree: self }
     }
 
     pub(crate) fn root_idx(&self) -> u32 {
@@ -312,7 +324,7 @@ impl<K: SyntaxKind> SyntaxTree<K> {
         }
     }
 
-    pub(crate) unsafe fn get_start_node(&self, idx: u32) -> (K, u32, u32, u32) {
+    pub(crate) unsafe fn get_start_node(&self, idx: u32) -> (C::NodeKind, u32, u32, u32) {
         let idx = idx as usize;
         debug_assert!(idx + START_NODE_SIZE as usize <= self.data.len());
 
@@ -322,12 +334,12 @@ impl<K: SyntaxKind> SyntaxTree<K> {
         let start = (ptr.add(6) as *const u32).read_unaligned();
         let end = (ptr.add(10) as *const u32).read_unaligned();
 
-        let kind = tag.get_start_node_kind();
+        let kind = tag.get_start_node_kind::<C>();
 
         (kind, finish_node_idx, start, end)
     }
 
-    pub(crate) unsafe fn get_add_token(&self, idx: u32) -> (K, u32, u32) {
+    pub(crate) unsafe fn get_add_token(&self, idx: u32) -> (C::TokenKind, u32, u32) {
         let idx = idx as usize;
         debug_assert!(idx + ADD_TOKEN_SIZE as usize <= self.data.len());
 
@@ -336,7 +348,7 @@ impl<K: SyntaxKind> SyntaxTree<K> {
         let start = (ptr.add(2) as *const u32).read_unaligned();
         let end = (ptr.add(6) as *const u32).read_unaligned();
 
-        let kind = tag.get_add_token_kind();
+        let kind = tag.get_add_token_kind::<C>();
 
         (kind, start, end)
     }
@@ -360,13 +372,13 @@ impl<K: SyntaxKind> SyntaxTree<K> {
     }
 }
 
-struct Events<'a, K> {
+struct Events<'a, C> {
     idx: u32,
-    tree: &'a SyntaxTree<K>,
+    tree: &'a SyntaxTree<C>,
 }
 
-impl<K: SyntaxKind> Iterator for Events<'_, K> {
-    type Item = Event<K>;
+impl<C: TreeConfig> Iterator for Events<'_, C> {
+    type Item = Event<C>;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.idx >= self.tree.data.len() as u32 {
@@ -374,17 +386,15 @@ impl<K: SyntaxKind> Iterator for Events<'_, K> {
         }
 
         if unsafe { self.tree.is_start_node(self.idx) } {
-            let (kind, _, start, end) = unsafe { self.tree.get_start_node(self.idx) };
-            let range = TextRange::new(start.into(), end.into());
+            let node = unsafe { SyntaxNode::new(self.idx, self.tree.id()) };
             self.idx += START_NODE_SIZE;
-            return Some(Event::StartNode { kind, range });
+            return Some(Event::StartNode(node));
         }
 
         if unsafe { self.tree.is_add_token(self.idx) } {
-            let (kind, start, end) = unsafe { self.tree.get_add_token(self.idx) };
-            let range = TextRange::new(start.into(), end.into());
+            let token = unsafe { SyntaxToken::new(self.idx, self.tree.id()) };
             self.idx += ADD_TOKEN_SIZE;
-            return Some(Event::AddToken { kind, range });
+            return Some(Event::AddToken(token));
         }
 
         if unsafe { self.tree.is_finish_node(self.idx) } {
@@ -396,20 +406,69 @@ impl<K: SyntaxKind> Iterator for Events<'_, K> {
     }
 }
 
+struct RawEvents<'a, C> {
+    idx: u32,
+    tree: &'a SyntaxTree<C>,
+}
+
+impl<C: TreeConfig> Iterator for RawEvents<'_, C> {
+    type Item = RawEvent<C>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.idx >= self.tree.data.len() as u32 {
+            return None;
+        }
+
+        if unsafe { self.tree.is_start_node(self.idx) } {
+            let (kind, _, start, end) = unsafe { self.tree.get_start_node(self.idx) };
+            let range = TextRange::new(start.into(), end.into());
+            self.idx += START_NODE_SIZE;
+            return Some(RawEvent::StartNode { kind, range });
+        }
+
+        if unsafe { self.tree.is_add_token(self.idx) } {
+            let (kind, start, end) = unsafe { self.tree.get_add_token(self.idx) };
+            let range = TextRange::new(start.into(), end.into());
+            self.idx += ADD_TOKEN_SIZE;
+            return Some(RawEvent::AddToken { kind, range });
+        }
+
+        if unsafe { self.tree.is_finish_node(self.idx) } {
+            self.idx += FINISH_NODE_SIZE;
+            return Some(RawEvent::FinishNode);
+        }
+
+        unreachable!()
+    }
+}
+
 /// The events in a syntax tree, as emitted by [`SyntaxTree::events`].
-///
-/// All data here is exactly as it is stored in the tree, with nothing extra computed.
+/// See that method’s documentation for more.
 #[derive(Clone, Copy, PartialEq, Eq, Hash)]
-pub enum Event<K> {
+pub enum Event<C> {
     #[allow(missing_docs)]
-    StartNode { kind: K, range: TextRange },
+    StartNode(SyntaxNode<C>),
     #[allow(missing_docs)]
-    AddToken { kind: K, range: TextRange },
+    AddToken(SyntaxToken<C>),
     #[allow(missing_docs)]
     FinishNode,
 }
 
-impl<K: SyntaxKind> fmt::Debug for SyntaxTree<K> {
+/// The events in a syntax tree, as emitted by [`SyntaxTree::raw_events`].
+/// See that method’s documentation for more.
+///
+/// All data here is exactly as it is stored in the tree, with nothing extra computed.
+#[derive(Clone, Copy, PartialEq, Eq, Hash)]
+pub enum RawEvent<C: TreeConfig> {
+    #[allow(missing_docs)]
+    StartNode { kind: C::NodeKind, range: TextRange },
+    #[allow(missing_docs)]
+    AddToken { kind: C::TokenKind, range: TextRange },
+    #[allow(missing_docs)]
+    FinishNode,
+}
+
+impl<C: TreeConfig> fmt::Debug for SyntaxTree<C> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         if !f.alternate() {
             return f.debug_struct("SyntaxTree").field("data", &self.data).finish();
@@ -417,46 +476,35 @@ impl<K: SyntaxKind> fmt::Debug for SyntaxTree<K> {
 
         let mut indentation_level = 0_usize;
 
-        let mut idx = self.root_idx();
-        while idx < self.data.len() as u32 {
-            if unsafe { self.is_finish_node(idx) } {
-                indentation_level -= 1;
-                idx += FINISH_NODE_SIZE;
-                continue;
+        for event in self.events() {
+            match event {
+                Event::StartNode(node) => {
+                    for _ in 0..indentation_level {
+                        write!(f, "  ")?;
+                    }
+                    indentation_level += 1;
+                    let kind = node.kind(self);
+                    let range = node.range(self);
+                    writeln!(f, "{kind:?}@{range:?}")?;
+                }
+                Event::AddToken(token) => {
+                    for _ in 0..indentation_level {
+                        write!(f, "  ")?;
+                    }
+                    let kind = token.kind(self);
+                    let range = token.range(self);
+                    let text = token.text(self);
+                    writeln!(f, "{kind:?}@{range:?} {text:?}")?;
+                }
+                Event::FinishNode => indentation_level -= 1,
             }
-
-            for _ in 0..indentation_level {
-                write!(f, "  ")?;
-            }
-
-            if unsafe { self.is_start_node(idx) } {
-                let node = SyntaxNode::new(idx, self.id());
-                let kind = node.kind(self);
-                let range = node.range(self);
-                writeln!(f, "{kind:?}@{range:?}")?;
-                indentation_level += 1;
-                idx += START_NODE_SIZE;
-                continue;
-            }
-
-            if unsafe { self.is_add_token(idx) } {
-                let token = SyntaxToken::new(idx, self.id());
-                let kind = token.kind(self);
-                let text = token.text(self);
-                let range = token.range(self);
-                writeln!(f, "{kind:?}@{range:?} {text:?}")?;
-                idx += ADD_TOKEN_SIZE;
-                continue;
-            }
-
-            unreachable!()
         }
 
         Ok(())
     }
 }
 
-impl<K: SyntaxKind> fmt::Debug for Event<K> {
+impl<C: TreeConfig> fmt::Debug for RawEvent<C> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::StartNode { kind, range } => write!(f, "START_NODE {kind:?} {range:?}"),
@@ -471,15 +519,31 @@ mod tests {
     use super::*;
     use expect_test::expect;
 
-    #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
-    #[repr(u16)]
-    enum SyntaxKind {
+    #[derive(Debug, PartialEq)]
+    enum NodeKind {
         Root,
-        Arrow,
         Block,
+        Function,
+        __Last,
+    }
+
+    unsafe impl crate::SyntaxKind for NodeKind {
+        const LAST: u16 = Self::__Last as u16;
+
+        fn to_raw(self) -> u16 {
+            self as u16
+        }
+
+        unsafe fn from_raw(raw: u16) -> Self {
+            std::mem::transmute(raw as u8)
+        }
+    }
+
+    #[derive(Debug, PartialEq)]
+    enum TokenKind {
+        Arrow,
         Comment,
         FncKw,
-        Function,
         Ident,
         LBrace,
         LetKw,
@@ -488,7 +552,7 @@ mod tests {
         __Last,
     }
 
-    unsafe impl crate::SyntaxKind for SyntaxKind {
+    unsafe impl crate::SyntaxKind for TokenKind {
         const LAST: u16 = Self::__Last as u16;
 
         fn to_raw(self) -> u16 {
@@ -496,10 +560,18 @@ mod tests {
         }
 
         unsafe fn from_raw(raw: u16) -> Self {
-            std::mem::transmute(raw)
+            std::mem::transmute(raw as u8)
         }
     }
 
+    #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+    enum TreeConfig {}
+
+    impl crate::TreeConfig for TreeConfig {
+        type NodeKind = NodeKind;
+        type TokenKind = TokenKind;
+    }
+
     enum D {
         U16(u16),
         U32(u32),
@@ -508,7 +580,7 @@ mod tests {
 
     fn check<const N: usize>(
         input: &str,
-        f: impl Fn(&mut SyntaxBuilder<SyntaxKind>),
+        f: impl Fn(&mut SyntaxBuilder<TreeConfig>),
         data: [D; N],
     ) {
         let mut builder = SyntaxBuilder::new(input);
@@ -528,20 +600,20 @@ mod tests {
         assert_eq!(tree.data[4..], data);
     }
 
-    fn big_tree() -> SyntaxTree<SyntaxKind> {
+    fn big_tree() -> SyntaxTree<TreeConfig> {
         let mut builder = SyntaxBuilder::new("# foo\nfncbar->{};");
 
-        builder.start_node(SyntaxKind::Root);
-        builder.add_token(SyntaxKind::Comment, TextRange::new(0.into(), 6.into()));
-        builder.start_node(SyntaxKind::Function);
-        builder.add_token(SyntaxKind::FncKw, TextRange::new(6.into(), 9.into()));
-        builder.add_token(SyntaxKind::Ident, TextRange::new(9.into(), 12.into()));
-        builder.add_token(SyntaxKind::Arrow, TextRange::new(12.into(), 14.into()));
-        builder.start_node(SyntaxKind::Block);
-        builder.add_token(SyntaxKind::LBrace, TextRange::new(14.into(), 15.into()));
-        builder.add_token(SyntaxKind::RBrace, TextRange::new(15.into(), 16.into()));
+        builder.start_node(NodeKind::Root);
+        builder.add_token(TokenKind::Comment, TextRange::new(0.into(), 6.into()));
+        builder.start_node(NodeKind::Function);
+        builder.add_token(TokenKind::FncKw, TextRange::new(6.into(), 9.into()));
+        builder.add_token(TokenKind::Ident, TextRange::new(9.into(), 12.into()));
+        builder.add_token(TokenKind::Arrow, TextRange::new(12.into(), 14.into()));
+        builder.start_node(NodeKind::Block);
+        builder.add_token(TokenKind::LBrace, TextRange::new(14.into(), 15.into()));
+        builder.add_token(TokenKind::RBrace, TextRange::new(15.into(), 16.into()));
         builder.finish_node();
-        builder.add_token(SyntaxKind::Semicolon, TextRange::new(16.into(), 17.into()));
+        builder.add_token(TokenKind::Semicolon, TextRange::new(16.into(), 17.into()));
         builder.finish_node();
         builder.finish_node();
 
@@ -553,12 +625,12 @@ mod tests {
         check(
             "",
             |b| {
-                b.start_node(SyntaxKind::Root);
+                b.start_node(NodeKind::Root);
                 b.finish_node();
             },
             [
                 D::U32(0),
-                D::U16(SyntaxKind::Root as u16 | 1 << 15),
+                D::U16(NodeKind::Root as u16 | 1 << 15),
                 D::U32(22),
                 D::U32(0),
                 D::U32(0),
@@ -572,18 +644,18 @@ mod tests {
         check(
             "let",
             |b| {
-                b.start_node(SyntaxKind::Root);
-                b.add_token(SyntaxKind::LetKw, TextRange::new(0.into(), 3.into()));
+                b.start_node(NodeKind::Root);
+                b.add_token(TokenKind::LetKw, TextRange::new(0.into(), 3.into()));
                 b.finish_node();
             },
             [
                 D::U32(3),
                 D::Text("let"),
-                D::U16(SyntaxKind::Root as u16 | 1 << 15),
+                D::U16(NodeKind::Root as u16 | 1 << 15),
                 D::U32(35),
                 D::U32(0),
                 D::U32(3),
-                D::U16(SyntaxKind::LetKw as u16),
+                D::U16(TokenKind::LetKw as u16),
                 D::U32(0),
                 D::U32(3),
                 D::U16(u16::MAX),
@@ -593,8 +665,8 @@ mod tests {
 
     #[test]
     fn debug_empty() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
         builder.finish_node();
 
         let tree = builder.finish();
@@ -623,6 +695,38 @@ mod tests {
 
     #[test]
     fn events() {
+        let tree = big_tree();
+        let mut events = tree.events();
+
+        let root = match events.next() {
+            Some(Event::StartNode(root)) => root,
+            _ => unreachable!(),
+        };
+        assert_eq!(root.kind(&tree), NodeKind::Root);
+
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::StartNode(_))));
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::StartNode(_))));
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::AddToken(_))));
+        assert!(matches!(events.next(), Some(Event::FinishNode)));
+
+        let semicolon = match events.next() {
+            Some(Event::AddToken(semicolon)) => semicolon,
+            _ => unreachable!(),
+        };
+        assert_eq!(semicolon.kind(&tree), TokenKind::Semicolon);
+
+        assert!(matches!(events.next(), Some(Event::FinishNode)));
+        assert!(matches!(events.next(), Some(Event::FinishNode)));
+        assert!(matches!(events.next(), None));
+    }
+
+    #[test]
+    fn raw_events() {
         expect![[r#"
             [
                 START_NODE Root 0..17,
@@ -640,32 +744,32 @@ mod tests {
                 FINISH_NODE,
             ]
         "#]]
-        .assert_debug_eq(&big_tree().events().collect::<Vec<_>>());
+        .assert_debug_eq(&big_tree().raw_events().collect::<Vec<_>>());
     }
 
     #[test]
     #[should_panic(expected = "no nodes are yet to be finished")]
     fn no_start_node() {
-        let mut builder = SyntaxBuilder::<SyntaxKind>::new("");
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
         builder.finish_node();
     }
 
     #[test]
     #[should_panic(expected = "did not finish all nodes (1 unfinished nodes)")]
     fn no_finish_node() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
         builder.finish();
     }
 
     #[test]
     #[should_panic(expected = "did not finish all nodes (2 unfinished nodes)")]
     fn too_many_start_node_calls() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
-        builder.start_node(SyntaxKind::Function);
-        builder.start_node(SyntaxKind::Block);
-        builder.start_node(SyntaxKind::Block);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
+        builder.start_node(NodeKind::Function);
+        builder.start_node(NodeKind::Block);
+        builder.start_node(NodeKind::Block);
         builder.finish_node();
         builder.finish_node();
         builder.finish();
@@ -674,10 +778,10 @@ mod tests {
     #[test]
     #[should_panic(expected = "no nodes are yet to be finished")]
     fn too_many_finish_node_calls() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
-        builder.start_node(SyntaxKind::Function);
-        builder.start_node(SyntaxKind::Block);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
+        builder.start_node(NodeKind::Function);
+        builder.start_node(NodeKind::Block);
         builder.finish_node();
         builder.finish_node();
         builder.finish_node();
@@ -687,37 +791,37 @@ mod tests {
     #[test]
     #[should_panic(expected = "root node already created")]
     fn second_root() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
         builder.finish_node();
-        builder.start_node(SyntaxKind::Block);
+        builder.start_node(NodeKind::Block);
     }
 
     #[test]
     #[should_panic(expected = "no nodes created")]
     fn empty_without_text() {
-        SyntaxBuilder::<SyntaxKind>::new("").finish();
+        SyntaxBuilder::<TreeConfig>::new("").finish();
     }
 
     #[test]
     #[should_panic(expected = "no nodes created")]
     fn empty_with_text() {
-        SyntaxBuilder::<SyntaxKind>::new("foo").finish();
+        SyntaxBuilder::<TreeConfig>::new("foo").finish();
     }
 
     #[test]
     #[should_panic(expected = "cannot add token before starting node")]
     fn add_token_before_starting_node() {
-        let mut builder = SyntaxBuilder::new("let");
-        builder.add_token(SyntaxKind::LetKw, TextRange::new(0.into(), 3.into()));
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("let");
+        builder.add_token(TokenKind::LetKw, TextRange::new(0.into(), 3.into()));
     }
 
     #[test]
     #[should_panic(expected = "token is out of range: range is 0..1, but text is 0..0")]
     fn add_token_with_out_of_bounds_range() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
-        builder.add_token(SyntaxKind::LetKw, TextRange::new(0.into(), 1.into()));
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
+        builder.add_token(TokenKind::LetKw, TextRange::new(0.into(), 1.into()));
     }
 
     #[test]
@@ -725,13 +829,13 @@ mod tests {
         expected = "tried to access node data from tree other than the one this node is from"
     )]
     fn access_node_data_from_other_tree() {
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
         builder.finish_node();
         let tree = builder.finish();
 
-        let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("");
+        builder.start_node(NodeKind::Root);
         builder.finish_node();
         let tree2 = builder.finish();
 
@@ -743,14 +847,14 @@ mod tests {
         expected = "tried to access token data from tree other than the one this token is from"
     )]
     fn access_token_data_from_other_tree() {
-        let mut builder = SyntaxBuilder::new("->");
-        builder.start_node(SyntaxKind::Root);
-        builder.add_token(SyntaxKind::Arrow, TextRange::new(0.into(), 2.into()));
+        let mut builder = SyntaxBuilder::<TreeConfig>::new("->");
+        builder.start_node(NodeKind::Root);
+        builder.add_token(TokenKind::Arrow, TextRange::new(0.into(), 2.into()));
         builder.finish_node();
         let tree = builder.finish();
 
         let mut builder = SyntaxBuilder::new("");
-        builder.start_node(SyntaxKind::Root);
+        builder.start_node(NodeKind::Root);
         builder.finish_node();
         let tree2 = builder.finish();
 
